from operator import lt, le, eq, ne, ge, gt
from typing import Literal

COMPARISON_OPERATORS_INFO = {
    "lt": {"operator": lt, "description": "Less than"},
    "le": {"operator": le, "description": "Less or equal"},
    "eq": {"operator": eq, "description": "Equal"},
    "ne": {"operator": ne, "description": "Not equal"},
    "ge": {"operator": ge, "description": "Greater or equal"},
    "gt": {"operator": gt, "description": "Greater than"},
}

type ComparisonOperatorsType = Literal["lt", "le", "eq", "ne", "ge", "gt"]


def compare[A, B](operator_type: ComparisonOperatorsType, a: A | B, b: A | B) -> bool:
    return COMPARISON_OPERATORS_INFO[operator_type]["operator"](a, b)



from .generate_secret import generate_secret, run as run_generate_secret
from .generate_htpasswd import generate_htpasswd, run as run_generate_htpasswd

__all__ = (
    "generate_secret",
    "run_generate_secret",
    "generate_htpasswd",
    "run_generate_htpasswd",
)



from getpass import getpass
from typing import Iterable

from .config import CLIConfig
from .operators_helper import ComparisonOperatorsType, COMPARISON_OPERATORS_INFO, compare


def print_divider(
    line_symbols: str = CLIConfig.line_symbols,
    line_length: int = CLIConfig.line_length,
) -> None:
    print(line_symbols * line_length)


def print_options(options: dict[str, str]) -> None:
    for key, value in options.items():
        print(f"{key}) {value}")


def do_comparison_operations[I](
    value: I,
    comparison_operations: Iterable[tuple[ComparisonOperatorsType, I]] = (),
) -> bool:
    results = [True]

    for operator, comparison_value in comparison_operations:
        result = compare(operator, value, comparison_value)

        if not result:
            desc = COMPARISON_OPERATORS_INFO[operator]["description"]

            print(f"Value must be {desc.lower()} {comparison_value!r}")

        results.append(result)

    return all(results)


def input_password() -> str:
    password = getpass("Input password: ")
    confirm = getpass("Confirm password: ")

    if password != confirm:
        print("❌ Passwords do not match.")
        exit(1)

    return password


def input_single[I](
    prompt: str,
    *,
    input_type: type[I],
    comparison_operations: Iterable[tuple[ComparisonOperatorsType, I]] = (),
    default_value: I | None = None,
) -> I:
    input_prompt = f"(default: {default_value!r}) > " if default_value is not None else "> "

    while True:
        print(prompt)

        try:
            value = input(input_prompt).strip()

            if value == "" and default_value is not None:
                value = default_value
                break

            value = input_type(value)
            check_compare = do_comparison_operations(value, comparison_operations)

            if check_compare:
                break

        except ValueError:
            print("--- Invalid input ---")

    print_divider()
    return value


def input_choices[I, R](
    cli_args: dict[I, R],
    options: dict[I, str],
    *,
    input_type: type[I],
    default_choice: I | None = None,
) -> R:
    print_options(options)
    input_prompt = f"(default: {default_choice!r}) > " if default_choice is not None else "> "

    while True:
        try:
            choice = input(input_prompt).strip()

            if choice == "" and default_choice is not None and default_choice in cli_args:
                choice = default_choice
                break

            choice = input_type(choice)

            if choice in cli_args:
                break

        except ValueError:
            print("--- Invalid choice ---")

        print("Please choose one of the following options:")
        print_options(options)

    print_divider()
    return cli_args[choice]



from os import getcwd
from pathlib import Path

from bcrypt import gensalt, hashpw

from .cli_helper import input_single, input_password
from .config import Config


def generate_htpasswd(
    username: str,
    password: str,
    htpasswd_file: Path,
    encoding: str = Config.encoding,
) -> None:
    """Create ore extend .htpasswd"""
    hashed = hashpw(password.encode(encoding), gensalt())
    entry = f"{username}:{hashed.decode()}\n"

    mode = "a" if htpasswd_file.exists() else "w"

    with htpasswd_file.open(mode, encoding=encoding) as f:
        f.write(entry)

    print(f"✅ User '{username}' add to {htpasswd_file.absolute().as_posix()}")


def username_menu() -> str:
    return input_single("Username:", input_type=str)


def password_menu() -> str:
    return input_password()


def menu() -> None:
    username = username_menu()
    password = password_menu()

    htpasswd = Path(getcwd()) / ".htpasswd"

    generate_htpasswd(username, password, htpasswd)


def run() -> None:
    try:
        menu()
    except KeyboardInterrupt:
        print("Cancelled")


if __name__ == "__main__":
    run()



from dataclasses import dataclass
from random import choices, shuffle
from string import punctuation, ascii_lowercase, ascii_uppercase, digits

from .cli_helper import input_choices, input_single
from .config import CLIConfig


@dataclass(frozen=True)
class Config:
    lowercase_count: int = 3
    uppercase_count: int = 3
    punctuation_count: int = 3
    digits_count: int = 3

    url_safe_punctuation: str = "-_.~"

    default_length: int = 40
    required_part_length: int = lowercase_count + uppercase_count + punctuation_count + digits_count
    max_length: int = 4096


def generate_secret(length: int, punctuation_symbols_sequence: str = punctuation) -> str:
    symbols = f"{punctuation_symbols_sequence}{ascii_lowercase}{ascii_uppercase}{digits}"

    if punctuation_symbols_sequence == "":
        punctuation_symbols = []
    else:
        punctuation_symbols = choices(punctuation_symbols_sequence, k=Config.punctuation_count)

    lowercase_symbols = choices(ascii_lowercase, k=Config.lowercase_count)
    uppercase_symbols = choices(ascii_uppercase, k=Config.uppercase_count)
    digits_symbols = choices(digits, k=Config.digits_count)

    required_part = (
        f"{"".join(lowercase_symbols + uppercase_symbols + punctuation_symbols + digits_symbols)}"
    )

    if length < len(required_part):
        raise ValueError("Password length is too short")

    base_pass_len = length - len(required_part)
    base_part = "".join(choices(symbols, k=base_pass_len))

    total = f"{required_part}{base_part}"
    total_list = list(total)

    shuffle(total_list)
    return "".join(total_list)


def punctuation_menu() -> str | None:
    cli_args = {1: Config.url_safe_punctuation, 2: punctuation, 3: ""}
    options = {
        1: "URL safe punctuation",
        2: "All punctuation symbols",
        3: "No punctuation symbols",
    }
    return input_choices(cli_args, options, input_type=int, default_choice=2)


def password_menu() -> int:
    return input_single(
        f"Length of password ({Config.required_part_length} - {Config.max_length}):",
        input_type=int,
        comparison_operations=[
            ("ge", Config.required_part_length),
            ("le", Config.max_length),
        ],
        default_value=Config.default_length,
    )


def menu() -> None:
    punctuation_symbols = punctuation_menu()
    password_length = password_menu()

    password = generate_secret(password_length, punctuation_symbols)

    top = f"{' Secret ':{CLIConfig.line_symbols}^{CLIConfig.line_length}}"
    bottom = CLIConfig.line_symbols * CLIConfig.line_length

    print(top, password, bottom, sep="\n")


def run() -> None:
    try:
        menu()
    except KeyboardInterrupt:
        print("Cancelled")


if __name__ == "__main__":
    run()
